=begin rdoc
 = jimpachi.rb - gui event logic for jimpachi

Author:: Kuzuno Hiroki, Nakai Yushi
Copyright:: Copyright (c) 2004 Kuzuno Hiroki, Nakai Yushi
License:: GNU General Public License version 2, or any later.
=end

require 'player'
require 'jimpachi_core'

class Jimpachi < JimpachiCore

  def initialize
    super() # create window and other compornent
    @music_name= nil
    $music_play= Proc.new{
      @player.play( @music_name ) unless @music_name == nil
    }
    @right_menu= set_right_menu()
    set_window_signal()
    set_button_signal()
    set_add_fs_signal()
    set_list_fs_signal()
    set_treeview_signal(0)
    set_notebook_signal()
    set_srand()
    @window.show
  end

  # set pseudo random 
  def set_srand()
    srand( ((Time.new.to_f * Process.pid).to_i >> 15))
  end

  # set main window signal
  def set_window_signal
    @window.signal_connect("delete_event") do 
      Gtk.main_quit 
      true 
    end
    @window.signal_connect("delete_event") do
      Gtk.main_quit
    end
  end
  
  # set jimpachi tab notebook signal
  def set_notebook_signal
    @notebook.signal_connect("button_press_event") do |tree, event|
      if event.button == 3
        @right_menu.popup(event.x_root, event.y_root, event.button, event.time)
      end
    end

    @notebook.signal_connect("switch-page") do |notebook, page, num|
      set_current_treeview(num)
    end
  end

  # set add music file selection signal
  def set_add_fs_signal
    @add_fs.ok_button.signal_connect("clicked") do
      puts "Selected filename: #{@add_fs.filename}"
      @player.add_music( @add_fs.filename )
      @add_fs.hide
      show_tree
    end
    @add_fs.cancel_button.signal_connect("clicked") do
      @add_fs.hide
    end
  end

  # set play list file selection signal
  def set_list_fs_signal
    @list_fs.ok_button.signal_connect("clicked") do
      puts "Selected filename: #{@list_fs.filename}"
      @player.add_list( @list_fs.filename )
      @list_fs.hide
      show_tree
    end
    @list_fs.cancel_button.signal_connect("clicked") do
      @list_fs.hide
    end
  end

  # set each button event
  def set_button_signal
    @prev_button.signal_connect("clicked") do 
      @player.prev_music
    end

    @play_button.signal_connect("clicked") do 
      $music_play.call
    end

    @pause_button.signal_connect("clicked") do 
      @player.pause  
    end

    @stop_button.signal_connect("clicked") do 
      @player.stop
    end

    @next_button.signal_connect("clicked") do 
      @player.next_music
    end
  end
  
  # set playlist treeview signal
  def set_treeview_signal( page_num="0" )
    treeview= @notebook.get_nth_page( page_num )

    treeview.signal_connect("cursor-changed") do |tree|
      @music_name= @model.get_value( @model.get_iter( tree.cursor[0] ), 2) unless @model.get_iter( tree.cursor[0] ) == nil
    end

    treeview.signal_connect("row-activated") do |tree, path, column|
      $music_play.call
    end

    treeview.signal_connect("button_press_event") do |tree, event|
      if event.button == 3
        @right_menu.popup(event.x_root, event.y_root, event.button, event.time)
      end
    end
    
  end

  # set treeview menu item factory
  def set_right_menu
    accelgroup = Gtk::AccelGroup.new
    @window.add_accel_group(accelgroup)
    ifp = Gtk::ItemFactory.new(Gtk::ItemFactory::TYPE_MENU, "<sub>", accelgroup)
    ifp.create_item("/New Tab", "<StockItem>", "<control>T", Gtk::Stock::COPY ,0) do 
      create_tab()
      set_treeview_signal( @notebook.page )
    end
    ifp.create_item("/Close Tab", "<StockItem>", "<control>W", Gtk::Stock::STOP, 0) do close_tab() end
    ifp.create_item("/separator", "<Separator>", nil, nil)

    ifp.create_item("/_Shuffle", "<Branch>", nil, nil)
    ifp.create_item("/_Shuffle/Current Tab", "<CheckItem>", nil, nil ,0) do  @player.tab_shuffle= set_option( @player.tab_shuffle ) end
    ifp.create_item("/_Shuffle/All Tabs", "<CheckItem>", nil, nil, 1) do @player.all_shuffle= set_option( @player.all_shuffle ) end

    ifp.create_item("/_Repeat", "<Branch>", nil, nil)
    ifp.create_item("/_Repeat/Selected File", "<CheckItem>", nil, nil ,0) do @player.one_repeat= set_option( @player.one_repeat ) end
     ifp.create_item("/_Repeat/Current Tab", "<CheckItem>", nil, nil ,0) do @player.tab_repeat= set_option( @player.tab_repeat ) end
    ifp.create_item("/_Repeat/All Tabs", "<CheckItem>", nil, nil, 1) do @player.all_reapet= set_option( @player.all_reapet ) end

    ifp.create_item("/separator", "<Separator>", nil, nil)
    ifp.create_item("/_Add", "<Branch>", "<alt>A", nil)
    ifp.create_item("/_Add/_File", "<StockItem>", "<control>F", Gtk::Stock::ADD, 1) do @add_fs.show end
    ifp.create_item("/_Add/_Dir", "<StockItem>", "<control>D", Gtk::Stock::OPEN, 2) do p "Add Dir" end
    ifp.create_item("/_Add/_List", "<StockItem>", "<control>L", Gtk::Stock::NEW, 3) do @list_fs.show end

    ifp.create_item("/_Del", "<Branch>", "<alt>D", nil)
    ifp.create_item("/_Del/_File", "<StockItem>", "<control>X", Gtk::Stock::CUT, 1) do 
      delete_music( @music_name ) unless @music_name == nil
    end
    ifp.create_item("/_Del/_All", "<StockItem>", "<control>C", Gtk::Stock::DELETE, 2) do @model.clear; @player.delete_all_music end

    ifp.create_item("/_Sel", "<Branch>", "<alt>S", nil)
    ifp.create_item("/_Sel/_All", "<StockItem>", "<control>A", Gtk::Stock::BOLD, 1) do p "Sel File" end

    ifp.create_item("/_Misc", "<Branch>", "<alt>M", nil)
    ifp.create_item("/_Misc/_About", "<Item>", nil, nil, 1) do 
      @about.show_all
    end
    ifp.create_item("/_Misc/_Quit", "<StockItem>", "<control>Q", Gtk::Stock::QUIT) do Gtk.main_quit  end

    return ifp
  end

  private
  def set_option( option )
    if option
      return false
    else
      return true
    end
  end

end
