=begin rdoc
 = jimpachi.rb - gui event logic for jimpachi

Author:: Kuzuno Hiroki, Nakai Yushi
Copyright:: Copyright (c) 2004 Kuzuno Hiroki, Nakai Yushi
License:: GNU General Public License version 2, or any later.
=end

require 'player'
require 'MP3Mixer'
require 'jimpachi_interface'
require 'thread'

class JimpachiCore < JimpachiIF

  def initialize
    super()
    @player = Player.new()
    @thread = auto_update_time()
    set_current_treeview()
  end

  # create new_tab to notebook
  def create_tab
    tv = set_treeview()
    tv.show_all
    @notebook.append_page(tv, nil)
    @notebook.set_show_tabs(true)
    @notebook.next_page()
    @player.set_new_playlist # create new playlist
    set_current_treeview( @notebook.page )
  end
  
  # close current tab
  def close_tab
    @notebook.remove_page( @notebook.page )
    @notebook.set_show_tabs(false) if @notebook.n_pages == 1
  end

  # create time auto update/reverse current music thread
  def auto_update_time
    Thread.new do
      loop do
        sleep(0.1)
        if MP3Mixer::playing?
          @now_play_bar.push( 1, convert_seconds_to_minutes( @player.get_current_music_current_time ))
          @total_play_bar.push( 1, convert_seconds_to_minutes( @player.get_current_music_total_time ))
          reverse_current_music_row
        end
      end
    end
  end
  
  # convert seconds format into minutes format
  # ex. 230 -> 3:50
  # ===Return:
  # minutes format
  def convert_seconds_to_minutes( seconds )
    result = seconds / 60
    remainder = seconds - result * 60
    return "#{result}:#{remainder}"
  end

  def reverse_current_music_row
    music_name= @player.get_current_music_name
    @model.each do |model, path, iter|
      if model.get_value( iter, 2 ) == music_name
        iter[1] = Gdk::Color.new(0, 0, 65535)
      else
        iter[1] = Gdk::Color.new(0, 0, 0)
      end
    end
  end

  # show music name on play list tree
  def show_tree
    @model.clear
    @player.each do |music_file|
      p music_file.get_music_name if $DEBUG
      iter = @model.append(nil)
      iter[0] = music_file.get_artist_name
      iter[2] = music_file.get_music_name
      iter[3] = convert_seconds_to_minutes( music_file.get_total_time )
    end
    @treeview.set_model(@model)
  end

  def set_current_treeview( page_num=0 )
    @treeview= @notebook.get_nth_page( page_num )
    @model = @treeview.model()
    @player.set_current_playlist( page_num )
  end

  def delete_music( music_name )
    @player.delete_music( music_name )
    show_tree
  end

end
