=begin rdoc
 = jimpachi_if.rb - gui interface for jimpachi

Author:: Kuzuno Hiroki, Nakai Yushi
Copyright:: Copyright (c) 2004 Kuzuno Hiroki, Nakai Yushi
License:: GNU General Public License version 2, or any later.
=end

require 'player'
require 'gtk2'
require 'jimpachi_about_dialog'

class JimpachiIF
  JIMPACHI_VERSION= "jiMPachi version 0.0.1"

  def initialize  
    @window = Gtk::Window.new
    @window.set_title(JIMPACHI_VERSION)
    @window.set_name("main window")
    @window.set_default_size(400, 150) 
    set_add_fs("Music File Selection")
    set_list_fs("Play List Selection")
    set_frame()
    @about= JimpachiAboutDialog.new( @window, JIMPACHI_VERSION )
    @window.show_all
  end

  # set musicfile file selection window
  def set_add_fs( title )
    @add_fs= Gtk::FileSelection.new( title )
  end

  # set playlist file selection window
  def set_list_fs( title )
    @list_fs= Gtk::FileSelection.new( title )
   end
  
  # set jimpachi playlist frame
  def set_frame
    buffer = "PlayList"
    frame = Gtk::Frame.new(buffer)
    frame.set_border_width(0)
    set_frame_vbox( frame )
    @window.add(frame)
  end

  # set vbox in frame which has top_part and under_part
  def set_frame_vbox( frame )
    vbox = Gtk::VBox.new(false, nil)

    vbox.set_border_width(0)
    vbox.pack_start(set_top_part(), true, true, 0)
    vbox.pack_start(set_under_part(), false, false, 0)
    frame.add(vbox)
  end

  # set top_part which is ScrolledWindow
  def set_top_part
    scrolled_window = Gtk::ScrolledWindow.new()
    scrolled_window.set_border_width(0)
    scrolled_window.set_policy(Gtk::POLICY_AUTOMATIC,
                               Gtk::POLICY_AUTOMATIC)
    @notebook= set_notebook()

    scrolled_window.add_with_viewport(@notebook)
    return scrolled_window
  end

  # set tab notebook in scrolled window
  def set_notebook
    notebook = Gtk::Notebook.new()
    notebook.set_border_width(0)
    notebook.set_show_tabs( false )
    notebook.append_page(set_treeview(), nil)
    return notebook
  end

  # set playlist treeview in note book
  def set_treeview
    model = Gtk::TreeStore.new(String, Gdk::Color, String, String)
    treeview = Gtk::TreeView.new(model)
    treeview.set_border_width(0)
    treeview.set_headers_visible( false )

    renderer = Gtk::CellRendererText.new
    column = Gtk::TreeViewColumn.new("Artist", renderer, {
                                       :text => 0,
                                       :foreground_gdk => 1
                                   })
    treeview.append_column(column)
    # column 2 music title name
    column = Gtk::TreeViewColumn.new("Title", renderer, {
                                       :text => 2
                                       })
    treeview.append_column(column)
    # column 3 music total time
    column = Gtk::TreeViewColumn.new("Time", renderer, {
                                       :text => 3
                                   })
    treeview.append_column(column)
    return treeview
  end

  # set under_part
  def set_under_part
    # button part
    vbox = Gtk::VBox.new(true, nil)
    vbox.set_border_width(0)
    vbox.set_spacing(0)
    vbox.pack_start(set_button_box(), false, false, 0)

    # left menubar and right time status bar
    hbox = Gtk::HBox.new(true, nil)
    hbox.set_border_width(0)
    hbox.set_spacing(0)
    hbox.pack_start(set_menubar(), false, false, 0)
    hbox.pack_start(set_now_play_status(), false, true, 0)
    hbox.pack_start(set_total_play_status(), false, true, 0)
    
    vbox.pack_start(hbox, false ,false, 0)
    return vbox
  end

  # jimpachi control current music button
  def set_button_box
    box = Gtk::HButtonBox.new()
    box.set_border_width(0)
    box.set_spacing(0)

    @prev_button = Gtk::Button.new("Prev")
    @play_button = Gtk::Button.new("Play")
    @pause_button = Gtk::Button.new("Pause")
    @stop_button = Gtk::Button.new("Stop")
    @next_button = Gtk::Button.new("Next")

    box.pack_start(@prev_button, false, false, 0)
    box.pack_start(@play_button, false, false, 0)
    box.pack_start(@pause_button, false, false, 0)
    box.pack_start(@stop_button, false, false, 0)
    box.pack_start(@next_button, false, false, 0)
    return box
  end

  # jimpachi control list menu
  def set_menubar
    accelgroup = Gtk::AccelGroup.new
    @window.add_accel_group(accelgroup)

    ifp = Gtk::ItemFactory.new(Gtk::ItemFactory::TYPE_MENU_BAR, "<main>", accelgroup)
    ifp.create_item("/_Add", "<Branch>", "<alt>A", nil)
    ifp.create_item("/_Add/_File", "<StockItem>", "<control>F", Gtk::Stock::ADD, 1) do @add_fs.show end
    ifp.create_item("/_Add/_Dir", "<StockItem>", "<control>D", Gtk::Stock::OPEN, 2) do p "Add Dir" end
    ifp.create_item("/_Add/_List", "<StockItem>", "<control>L", Gtk::Stock::NEW, 3) do @list_fs.show end

    ifp.create_item("/_Del", "<Branch>", "<alt>D", nil)
    ifp.create_item("/_Del/_File", "<StockItem>", "<control>X", Gtk::Stock::CUT, 1) do 
      delete_music( @music_name ) unless @music_name == nil
    end
    ifp.create_item("/_Del/_All", "<StockItem>", "<control>C", Gtk::Stock::DELETE, 2) do @model.clear; @player.delete_all_music end

    ifp.create_item("/_Sel", "<Branch>", "<alt>S", nil)
    ifp.create_item("/_Sel/_All", "<StockItem>", "<control>A", Gtk::Stock::BOLD, 1) do p "Sel File" end

    ifp.create_item("/_Misc", "<LastBranch>", "<alt>M", nil)
    ifp.create_item("/_Misc/_About", "<Item>", nil, nil, 1) do 
      @about.show_all
    end
    ifp.create_item("/_Misc/_Quit", "<StockItem>", "<control>Q", Gtk::Stock::QUIT) do Gtk.main_quit  end

    @menu = ifp.get_widget("<main>")
    return ifp.get_widget("<main>")
  end

  # jimpachi display current/total music time
  def set_now_play_status
    @now_play_bar = Gtk::Statusbar.new()
    return @now_play_bar
  end

  def set_total_play_status
    @total_play_bar = Gtk::Statusbar.new()
    return @total_play_bar
  end

end
