=begin rdoc
 = player.rb - Jimpachi core

Author:: Kuzuno Hiroki, Nakai Yushi
Copyright:: Copyright (c) 2004 Kuzuno Hiroki, Nakai Yushi
License:: GNU General Public License version 2, or any later.

=end

require 'play_list'
require 'MP3Mixer'
require 'thread'

class Player

  def initialize
    @one_repeat= false
    @tab_repeat= false
    @all_repeat= false

    @tab_shuffle= false
    @all_shuffle= false

    @playlists= []
    @music_file= nil
    @loop= 0
    @now_playlist= nil
    @thread = nil
    set_new_playlist
  end

  attr_accessor(:one_repeat, :tab_repeat, :all_repeat,
       :tab_shuffle, :all_shuffle)

  # play next music file in current playlist
  # ===Return:
  # @music_file.get_music_name: if playlist is not end
  # next music file title.
  # flase on playlist index is end.
  def next_music
    if @tab_shuffle
      shuffle_music()
    elsif @music_file= @now_playlist.get_next
      play_music_file
      return @music_file.get_music_name
    end
    return false
  end

  # play prev music file in current playlist
  # ===Return:
  # @music_file.get_music_name: if playlist is not top
  # next music file title.
  # false on index in playlist is top.
  def prev_music
    if @tab_shuffle
      shuffle_music()
    elsif @music_file= @now_playlist.get_prev
      play_music_file
      return @music_file.get_music_name
    end
    return false
  end

  # play music file
  # ===Parameters:
  # * music_name: music_file title in playlist.
  def play( music_name )
    index = 0
    @music_files= @now_playlist.get_list
    @music_files.each do |music_file|
      if music_file.get_music_name == music_name
        @now_playlist.set_index( index )
        @music_file = music_file
        play_music_file
        break
      end
      index += 1
    end
  end

  def stop
    if MP3Mixer::playing? 
      @music_file.stop
      @thread.kill
    end
  end

  def pause
    @music_file.pause unless @music_file == nil
  end

  def seek
  end

  # each music_files in playlist
  # ===Return:
  # false on playlist/music_files is nil, 
  # iterator on music_files is MusicFile class
  def each
    music_files= @now_playlist.get_list
    unless music_files == nil
      music_files.each do |music_file|
        yield( music_file )
      end
    else
      return false
    end
  end

  # Add new jimpachilist
  # ===Parameters:
  # * list_name: list file path
  def add_list( list_name )
    @now_playlist.add_list( list_name )
  end

  # Add new music file
  # ===Parameters:
  # * file_path: music file path.
  def add_music( file_path )
    file_path_split= file_path.split("/")
    @now_playlist.add_music_file(file_path_split[-1], file_path)
  end

  # return current play music file name.
  def get_current_music_name
    return @music_file.get_music_name
  end

  def get_current_music_total_time
    return @music_file.get_total_time
  end

  def get_current_music_current_time
    return @music_file.get_current_time
  end

  # set current selected playlist tab
  def set_current_playlist( playlist_index )
    @now_playlist= @playlists[playlist_index]
  end

  # create new playlist
  def set_new_playlist
    @playlists.push( PlayList.new() )
  end

  def delete_music( music_name )
    @now_playlist.delete_music_file( music_name )
  end

  def delete_all_music()
    @now_playlist.delete_list()
  end

  private
  # if current play music exit, auto play next music 
  def auto_next
    Thread.new do
      loop do
        sleep(0.1)
        check_next() unless MP3Mixer::playing? 
      end
    end
  end

  # play music file
  def play_music_file
    stop
    @music_file.play(@loop)
    @thread=  auto_next() if @thread == nil
  end

  # check shuffle/repeat flag, play correct next music
  def check_next
    if @one_repeat
      play_music_file() 
    elsif @tab_repeat
      if @now_playlist.list_end?
        @now_playlist.set_index( -1 )
        next_music()
      end
    elsif @all_repeat
      p "all repeat is not implemented"
    elsif @tab_shuffle
      shuffle_music()
    elsif @all_shuffle
      p "all shuffle is not implemented"
    else
      next_music()
    end
  end

  # play shuffle music
  def shuffle_music
    @music_file= @now_playlist.get_random_music()
    play_music_file()
  end

end
